# -*- coding: utf-8 -*-
##############################################################################
#
#    wms_location module for OpenERP, Location management
#    Copyright (C) 2010 SYLEAM (<http://www.syleam.fr/>)
#              Christophe CHAUVET <christophe.chauvet@syleam.fr>
#
#    This file is a part of wms_location
#
#    wms_location is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    wms_location is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields


_auto_move = [
    ('auto', 'Automatic Move'),
    ('manual', 'Manual Operation'),
    ('transparent', 'Automatic No Step Added')
]


class product_location(osv.osv):
    _name = 'product.location'
    _description = 'Product per location'

    _columns = {
        'name': fields.char('Operation', size=64, required=True),
        'product_id': fields.many2one('product.product', 'Products', ondelete='cascade'),
        'location_from_id': fields.many2one('stock.location', 'Source Location', ondelete='cascade', required=True),
        'location_dest_id': fields.many2one('stock.location', 'Destination Location', ondelete='cascade', required=True),
        'delay': fields.integer('Delay (days)', help="Number of days to do this transition"),
        'auto': fields.selection(_auto_move, 'Automatic Move', required=True,
            help="This is used to define paths the product has to follow within the location tree.\n" \
                "The 'Automatic Move' value will create a stock move after the current one that will be "\
                "validated automatically. With 'Manual Operation', the stock move has to be validated "\
                "by a worker. With 'Automatic No Step Added', the location is replaced in the original move."
            ),
    }

    _defaults = {
        'auto': lambda *arg: 'auto',
        'delay': lambda *arg: 0,
    }

product_location()

_loc_type = [
    ('fixed', 'Fixed'),
    ('compute', 'Compute'),
    ('crossdock', 'Cross Docking'),
]


class product_product(osv.osv):
    _inherit = 'product.product'

    _columns = {
        'location_type': fields.selection(_loc_type,
                                          'Location type'),
        'location_ids': fields.one2many('product.location', 'product_id',
            'Location Paths',
            help="These rules set the right path of the product in the "\
            "whole location tree.")
    }

    _defaults = {
         'location_type': lambda *a: 'compute',
    }

product_product()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
