# -*- coding: utf-8 -*-
##############################################################################
#
#    wms_purchase module for OpenERP, Inherite purcharge for wms
#    Copyright (C) 2010 SYLEAM Info Services (<http://www.syleam.fr/>)
#              Jean-Sébastien SUZANNE <jean-sebastien.suzanne@syleam.fr>
#
#    This file is a part of wms_purchase
#
#    wms_purchase is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    wms_purchase is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
import pooler
import thread
from tools.translate import _
import netsvc
logger = netsvc.Logger()


class stock_picking(osv.osv):
    _inherit = 'stock.picking'


    def create(self, cr, uid, values, context=None):
        """
        overload create method to put warehouse_id on stock.picking
            if we've got a sale_order related to this stock_picking, so we take the warehouse of the sale_shop
            if we haven't got a sale_order related to this stock_picking, so we take the warehouse of the connected user
        """
        if context is None:
            context = {}

        sale_obj = self.pool.get('sale.order')
        purchase_obj = self.pool.get('purchase.order')
        user_obj = self.pool.get('res.users')

        ## SALE_ORDER TO STOCK_PICKING
        if values.get('sale_id') and values['sale_id']:
            # we've got a sale_order, so we take the warehouse of the sale_shop
            sale = sale_obj.browse(cr, uid, values['sale_id'], context=context)
            values['warehouse_id'] = sale.shop_id and sale.shop_id.warehouse_id and sale.shop_id.warehouse_id.id
        ## PURCHASE_ORDER TO STOCK_PICKING
        elif values.get('purchase_id') and values['purchase_id']:
            # we've got a sale_order, so we take the warehouse of the sale_shop
            purchase = purchase_obj.browse(cr, uid, values['purchase_id'], context=context)
            values['warehouse_id'] = purchase.warehouse_id and purchase.warehouse_id.id
        elif not values.get('sale_id', False) and values.get('origin', False):
            sale_ids = sale_obj.search(cr, uid, [('name', '=', values.get('origin'))], context=context)
            if not sale_ids:
                user = user_obj.browse(cr, uid, uid, context=context)
                values['warehouse_id'] = user.context_warehouse_id and user.context_warehouse_id.id
            elif len(sale_ids) > 1:
                raise osv.except_osv(_('ERROR'),_('%d sale orders found') % len(sale_ids))
            else:
                sale = sale_obj.browse(cr, uid, sale_ids[0], context=context)
                values['warehouse_id'] = sale.shop_id and sale.shop_id.warehouse_id and sale.shop_id.warehouse_id.id
        else:
            # we haven't got a sale_order, so we take the warehouse of the connected user
            user = user_obj.browse(cr, uid, uid, context=context)
            values['warehouse_id'] = user.context_warehouse_id and user.context_warehouse_id.id


        id = super(stock_picking, self).create(cr, uid, values, context=context)

        return id

    def compute_pump(self, cr, uid, id, product_linked=None, context=None):
        """
        this method id used to compute pump among this picking
        :type product_linked : dict
        :param product_linked : the key is the id of the product, the value is new the price_unit
        """
        if context is None:
            context = {}
        if product_linked is None:
            # make nothing
            return True
        request_obj = self.pool.get('res.request')
        product_obj = self.pool.get('product.product')
        move_obj = self.pool.get('stock.move')
        logger.notifyChannel('wms_purchase.stock_picking.compute_pump', netsvc.LOG_DEBUG, 'COMPUTE_PUMP START(id=%d, product_linked=%s, context=%s)' % (id, repr(product_linked), repr(context)))

        def thread_compute_pump(self, cr, uid, id, product_linked, context):
            """
            this method is a threadind method
            """
            cr = pooler.get_db(cr.dbname).cursor()
            # get information to display in message
            picking = self.browse(cr, uid, id, context)
            #for each move who have a product with a new price_unit compute_pump else make nothing
            for product_id in product_linked.keys():
                product = product_obj.browse(cr, uid, product_id, context=context)
                if product.cost_method != 'average':
                    continue
                try:
                    # find the move link with this product and this picking
                    move_ids = move_obj.search(cr, uid, [('picking_id', '=', picking.id), ('product_id', '=', product.id)], context=context)
                    if move_ids:
                        # we take the first because they can have more move but only one is needed to know if we must compute this product or not
                        move = move_obj.browse(cr, uid, move_ids[0], context=context)
                        if move.price_unit != product_linked[product.id]:
                            # save the new price unit one the move_ids
                            move_obj.write(cr, uid, move_ids, {'price_unit': product_linked[product.id]}, context=context)
                            # compute the move
                            move_obj.compute_pump(cr, uid, move_ids, context=context)
                        else:
                            continue
                    else:
                        raise osv.except_osv(_('ERROR'), _('No move found this picking(%s) and product(%s)')) % (picking.name, product.name)
                    #make a commit because the next product can be wrong and has an error
                    cr.commit()
                except Exception, err:
                    # make a rollback because the transaction is wrong
                    cr.rollback()
                    logger.notifyChannel('wms_purchase.stock_picking.thread_compute_pump', netsvc.LOG_ERROR, repr(err))
                    # make a request
                    val_request = {
                        'act_to': uid,
                        'act_from': uid,
                        'name': _('ERROR on the compute P.U.M.P.'),
                        'body': _("The picking (%s) at %s is on error \n\nThe product (%s) has not computed is P.U.M.P \n\nERROR : %s ") \
                                                                             % (picking.name, picking.min_date, product.name, repr(err)),
                        'priority': '2',
                        #'ref_doc1':'stock.picking,%d' % picking.id,
                        #'ref_doc2':'product.product,%d' % product.id,
                                }
                    request_obj.create(cr, uid, val_request, context=context)
                    cr.commit()
            logger.notifyChannel('wms_purchase.stock_picking.compute_pump', netsvc.LOG_INFO, 'COMPUTE_PUMP ENDED ....')
            cr.close()

        thread.start_new_thread(thread_compute_pump, (self, cr, uid, id, product_linked.copy(), context.copy()))
        return True

stock_picking()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
